// WordSnippet v1.0
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains the WordSnippet plug-in, which derives from the IDTExtensibility2 class. Essentially, it just registers a new taskbar with Outlook and/or Word (depending on the application
// settings), fills it with menus and buttons, registers some FileSystemWatcher objects, and connects everything to appropriate event handlers. The code is pretty messy - but it stems from the
// inherent messiness of COM and Office application programming. Let's hope Microsoft vastly cleans up this functionality in a future version of .NET!

namespace WordSnippet {

	#region Using declarations

	using System;
	using System.Collections;
	using System.Configuration;
	using Microsoft.Office.Core;
	using Microsoft.Office.Interop.Outlook;
	using Extensibility;
	using System.Runtime.InteropServices;
	using System.Reflection;

	#endregion

	[GuidAttribute("41226D93-A5F8-46CF-B4A6-81E2CC7630DB"), ProgId("WordSnippet.Connect")]
	public class Connect : Object, Extensibility.IDTExtensibility2	{

		#region Variables and initialization code

		private object applicationObject;
		private object addInInstance;
		private CommandBarPopup menu;
		private System.IO.FileSystemWatcher watcher;
		private string path;
		private bool bInstalled = false;
		object omissing = System.Reflection.Missing.Value;
		object oFalse;
		object oTrue;
		CommandBar snippetBar = null;
		Hashtable hash;

		public Connect() {  // constructor
			bool bFalse = false;
			oFalse = bFalse;
			bool bTrue = true;
			oTrue = bTrue;
			hash = new Hashtable();
		}

		#endregion

		#region COM interface functions (mostly uninteresting)

		public void OnConnection(object application, Extensibility.ext_ConnectMode connectMode, object addInInst, ref System.Array custom) {
			applicationObject = application;
			addInInstance = addInInst;
			if (connectMode != Extensibility.ext_ConnectMode.ext_cm_Startup)
				OnStartupComplete(ref custom);
		}

		public void OnDisconnection(Extensibility.ext_DisconnectMode disconnectMode, ref System.Array custom) {
			if (disconnectMode != Extensibility.ext_DisconnectMode.ext_dm_HostShutdown)
				OnBeginShutdown(ref custom);
			applicationObject = null;
		}

		public void OnAddInsUpdate(ref System.Array custom) {  // required but not used.
		}

		#endregion

		#region Core functions

		public void OnStartupComplete(ref System.Array custom) {
			// All of the real action happens here: Read configuration file settings, create menu bars and buttons, register button handlers, etc.
			bool bInstallWord = true;
			string strWordPath = "";
			bool bInstallOutlook = true;
			string strOutlookPath = "";
			if (System.IO.File.Exists(@"C:\Program Files\WordSnippet\WordSnippet.exe.config") == false) {  // use default settings
				if (System.IO.Directory.Exists(@"C:\Program Files\WordSnippet"))
					strWordPath = strOutlookPath = @"C:\Program Files\WordSnippet";
				else  // can't find snippets directory or config file - just exit. :(
					bInstallWord = bInstallOutlook = false;
			}
			else {  // read settings from configuration file
				System.IO.StreamReader file = new System.IO.StreamReader(@"C:\Program Files\WordSnippet\WordSnippet.exe.config");
				string strings = file.ReadToEnd();
				file.Close();
				// parse file
				if (strings.IndexOf("<add key=\"UseWordSnippets\" value=\"True\" />") > -1)
					bInstallWord = true;
				else if (strings.IndexOf("<add key=\"UseWordSnippets\" value=\"True\" />") > -1)
					bInstallWord = false;
				int startindex = strings.IndexOf("<add key=\"WordSnippets\" value=\"");
				if (startindex > -1) {   // found Word install path
					int endindex = strings.IndexOf("\"", startindex + 31);
					if (endindex > -1)
						strWordPath = strings.Substring(startindex + 31, endindex - startindex - 31);
				}
				if (strings.IndexOf("<add key=\"UseOutlookSnippets\" value=\"True\" />") > -1)
					bInstallOutlook = true;
				else if (strings.IndexOf("<add key=\"UseOutlookSnippets\" value=\"True\" />") > -1)
					bInstallOutlook = false;
				startindex = strings.IndexOf("<add key=\"OutlookSnippets\" value=\"");
				if (startindex > -1) {   // found Outlook install path
					int endindex = strings.IndexOf("\"", startindex + 34);
					if (endindex > -1)
						strOutlookPath = strings.Substring(startindex + 34, endindex - startindex - 34);
				}
			}

			CommandBars oCommandBars;
			if ((applicationObject is Microsoft.Office.Interop.Word.ApplicationClass) && (bInstallWord == true)) {
				// Install the plug-in immediately.
				path = strWordPath;
				try {
					oCommandBars = (CommandBars)applicationObject.GetType().InvokeMember("CommandBars", BindingFlags.GetProperty , null, applicationObject ,null);
					CreateSnippetBar(oCommandBars);
				} catch {}
			}
			else if ((applicationObject is Microsoft.Office.Interop.Outlook.Application) && (bInstallOutlook == true)) {
				// Hook the NewInspector object, and install the snippet bar in it when it's created.
				path = strOutlookPath;
				try {
					Microsoft.Office.Interop.Outlook.Inspectors inspectors = ((Microsoft.Office.Interop.Outlook.Application) applicationObject).Inspectors;
					inspectors.NewInspector += new InspectorsEvents_NewInspectorEventHandler(InstallSnippetBarInNewInspector);
				} catch {}
			}
			watcher = new System.IO.FileSystemWatcher(path);
			watcher.EnableRaisingEvents = true;
			watcher.Deleted += new System.IO.FileSystemEventHandler(FileSystemChanged);
			watcher.Created += new System.IO.FileSystemEventHandler(FileSystemChanged);
			watcher.Renamed += new System.IO.RenamedEventHandler(FileSystemRenamed);
		}

		private void InstallSnippetBarInNewInspector(Inspector inspector) {
			// This function installs the Snippet Bar in the Message pane of Microsoft Outlook.
			try {
				CreateSnippetBar(inspector.CommandBars);
			} catch {}
		}

		public void CreateSnippetBar(CommandBars oCommandBars) {
			// This function installs the Snippet Bar in Microsoft Word.
			menu = null;
			try {  // either find the old bar
				snippetBar = null;
				foreach (CommandBar c in oCommandBars) {
					if (c.Name == "WordSnippet")
						snippetBar = c;
				}
				if (snippetBar == null)
					snippetBar = oCommandBars.Add("WordSnippet", MsoBarPosition.msoBarTop, omissing, omissing);
			} catch {}
			try {  // delete all controls on bar
				while (snippetBar.Controls.Count > 0)
					snippetBar.Controls[1].Delete(oFalse);
			}  catch {}
			try {  // add new menu to bar and populate it
				menu = (CommandBarPopup) snippetBar.Controls.Add(MsoControlType.msoControlPopup, omissing, omissing, omissing, omissing);
				menu.Caption = "WordSnippet";
				menu.Tag = "WordSnippet";
				//menu.OnAction = "<WordSnippet.Connect>";   // this is recommended but always causes a bombout in Word. :(
				menu.Visible = true;
				FillMenu();
				bInstalled = true;
			} catch {}
		}

		private void FillMenu() {
			// This function removes all items in current menu in popup bar, and fills with new items.
			Random r = new Random((int) System.DateTime.Now.Ticks);
			try {
				RecursiveDelete(ref menu);
				RecursiveAdd(ref menu, path, r);
			} catch {}
		}

		private void RecursiveDelete(ref CommandBarPopup popup) {
			try {
				while (popup.Controls.Count > 0) {
					object o = popup.Controls[1];
					if (o is CommandBarPopup) {
						CommandBarPopup p = (CommandBarPopup) o;
						p.Tag = "";
						RecursiveDelete(ref p);
						p.Delete(oFalse);
					}
					else if (o is CommandBarButton) {
						CommandBarButton b = (CommandBarButton) o;
						hash.Remove(b.Tag);
						b.Delete(oFalse);
					}
				}
			} catch (SystemException ex) { System.Windows.Forms.MessageBox.Show(ex.Message);}
		}

		private void RecursiveAdd(ref CommandBarPopup popup, string path, Random r) {
			foreach (string directory in System.IO.Directory.GetDirectories(path)) {  // recursively add file folders as submenus.
				CommandBarPopup newPopup = (CommandBarPopup) popup.Controls.Add(MsoControlType.msoControlPopup, omissing, omissing, omissing, omissing);
				newPopup.Caption = directory.Substring(path.Length + 1);
				RecursiveAdd(ref newPopup, directory, r);
			}
			foreach (string file in System.IO.Directory.GetFiles(path, "*.doc")) {  // add all .doc files as buttons in this level of the hierarchial menu.
				if (file.StartsWith("~") == false) {
					CommandBarButton newButton = (CommandBarButton) popup.Controls.Add(MsoControlType.msoControlButton, omissing, omissing, omissing, omissing);
					newButton.Caption = file.Substring(path.Length + 1, file.Length - path.Length - 5);
					newButton.Click += new _CommandBarButtonEvents_ClickEventHandler(SnippetButton_Click);
					int tag = r.Next();
					while (hash.Contains(tag) == true)
						tag = r.Next();
					newButton.Tag = tag.ToString();
					hash.Add(tag.ToString(), file);
				}
			}
		}

		public void OnBeginShutdown(ref System.Array custom) {
			// Try to remove the SnippetBar from Microsoft Word.
			if (bInstalled) {
				menu.Delete(omissing);
				menu = null;
				snippetBar.Delete();
				snippetBar = null;
			}
		}

		private void SnippetButton_Click(CommandBarButton button, ref bool cancel) {
			string path = hash[button.Tag].ToString();  // find the filename associated with the button in the hashtable.
			if (System.IO.File.Exists(path)) {
				// load contents
				bool bFalse = false;
				object oFalse = bFalse;
				object oPath = path;
				Microsoft.Office.Interop.Word.Application newapp = new Microsoft.Office.Interop.Word.ApplicationClass();
				Microsoft.Office.Interop.Word.DocumentClass d = (Microsoft.Office.Interop.Word.DocumentClass) newapp.Documents.Open(ref oPath, ref omissing, ref oTrue, ref oFalse, ref omissing, ref omissing, ref oFalse, ref omissing, ref omissing, ref omissing, ref omissing, ref oFalse, ref omissing, ref omissing, ref omissing, ref omissing);
				string strSelection = d.Range(ref omissing, ref omissing).Text;
				strSelection = strSelection.Substring(0, strSelection.Length - 1);
				d.Close(ref oFalse, ref omissing, ref omissing);
				newapp.Quit(ref oFalse, ref omissing, ref omissing);
				// insert into new document
				try {
					// Microsoft Outlook
					Microsoft.Office.Interop.Outlook.Application app = (Microsoft.Office.Interop.Outlook.Application) applicationObject;
					Microsoft.Office.Interop.Outlook.Inspector i = app.ActiveInspector();
					object o = i.WordEditor;
					if (i.CurrentItem is Microsoft.Office.Interop.Outlook.MailItem) {
						MailItem m = (MailItem) i.CurrentItem;
						System.Windows.Forms.SendKeys.SendWait("`*!");
						int index = m.Body.IndexOf("`*!");
						if (index != -1)
							m.Body = m.Body.Substring(0, index) + strSelection + m.Body.Substring(index + 3);
					}
				} catch {
					// Microsoft Word
					Microsoft.Office.Interop.Word.Application app = (Microsoft.Office.Interop.Word.ApplicationClass) applicationObject;
					if (app.Selection.Range.Characters.Count > 1)
						app.Selection.Range.Delete(ref omissing, ref omissing);
					app.Selection.InsertBefore(strSelection);
					object oCollapse = Microsoft.Office.Interop.Word.WdCollapseDirection.wdCollapseEnd;
					app.Selection.Collapse(ref oCollapse);
				}
			}
		}

		private void FileSystemChanged(object sender, System.IO.FileSystemEventArgs e) {
			// On a change to the files in a watched menu (Creation or Deletion), update the Snippet Bar menu.
			FillMenu();
		}

		private void FileSystemRenamed(object sender, System.IO.RenamedEventArgs e) {
			// On a rename of a file in a watched menu, update the Snippet Bar menu.
			FillMenu();
		}

		#endregion

	}
}
